@@ -3,8 +3,8 @@
 //! `Table` is to WebAssembly tables what `LinearMemory` is to WebAssembly linear memories.
 
 use crate::vmcontext::{VMCallerCheckedAnyfunc, VMTableDefinition};
-use crate::{ResourceLimiter, Trap, VMExternRef};
-use anyhow::{bail, Result};
+use crate::{Store, Trap, VMExternRef};
+use anyhow::{bail, format_err, Error, Result};
 use std::convert::{TryFrom, TryInto};
 use std::ops::Range;
 use std::ptr;
@@ -137,11 +137,8 @@ fn wasm_to_table_type(ty: WasmType) -> Result<TableElementType> {
 
 impl Table {
     /// Create a new dynamic (movable) table instance for the specified table plan.
-    pub fn new_dynamic(
-        plan: &TablePlan,
-        limiter: Option<&mut dyn ResourceLimiter>,
-    ) -> Result<Self> {
-        Self::limit_new(plan, limiter)?;
+    pub fn new_dynamic(plan: &TablePlan, store: &mut dyn Store) -> Result<Self> {
+        Self::limit_new(plan, store)?;
         let elements = vec![0; plan.table.minimum as usize];
         let ty = wasm_to_table_type(plan.table.wasm_ty)?;
         let maximum = plan.table.maximum;
@@ -157,9 +154,9 @@ impl Table {
     pub fn new_static(
         plan: &TablePlan,
         data: &'static mut [usize],
-        limiter: Option<&mut dyn ResourceLimiter>,
+        store: &mut dyn Store,
     ) -> Result<Self> {
-        Self::limit_new(plan, limiter)?;
+        Self::limit_new(plan, store)?;
         let size = plan.table.minimum;
         let ty = wasm_to_table_type(plan.table.wasm_ty)?;
         let data = match plan.table.maximum {
@@ -170,14 +167,12 @@ impl Table {
         Ok(Table::Static { data, size, ty })
     }
 
-    fn limit_new(plan: &TablePlan, limiter: Option<&mut dyn ResourceLimiter>) -> Result<()> {
-        if let Some(limiter) = limiter {
-            if !limiter.table_growing(0, plan.table.minimum, plan.table.maximum) {
-                bail!(
-                    "table minimum size of {} elements exceeds table limits",
-                    plan.table.minimum
-                );
-            }
+    fn limit_new(plan: &TablePlan, store: &mut dyn Store) -> Result<()> {
+        if !store.table_growing(0, plan.table.minimum, plan.table.maximum)? {
+            bail!(
+                "table minimum size of {} elements exceeds table limits",
+                plan.table.minimum
+            );
         }
         Ok(())
     }
@@ -191,6 +186,7 @@ impl Table {
     }
 
     /// Returns whether or not the underlying storage of the table is "static".
+    #[cfg(feature = "pooling-allocator")]
     pub(crate) fn is_static(&self) -> bool {
         if let Table::Static { .. } = self {
             true
@@ -292,20 +288,22 @@ impl Table {
         &mut self,
         delta: u32,
         init_value: TableElement,
-        limiter: Option<&mut dyn ResourceLimiter>,
-    ) -> Option<u32> {
+        store: &mut dyn Store,
+    ) -> Result<Option<u32>, Error> {
         let old_size = self.size();
-        let new_size = old_size.checked_add(delta)?;
+        let new_size = match old_size.checked_add(delta) {
+            Some(s) => s,
+            None => return Ok(None),
+        };
 
-        if let Some(limiter) = limiter {
-            if !limiter.table_growing(old_size, new_size, self.maximum()) {
-                return None;
-            }
+        if !store.table_growing(old_size, new_size, self.maximum())? {
+            return Ok(None);
         }
 
         if let Some(max) = self.maximum() {
             if new_size > max {
-                return None;
+                store.table_grow_failed(&format_err!("Table maximum size exceeded"));
+                return Ok(None);
             }
         }
 
@@ -327,7 +325,7 @@ impl Table {
         self.fill(old_size, init_value, delta)
             .expect("table should not be out of bounds");
 
-        Some(old_size)
+        Ok(Some(old_size))
     }
 
     /// Get reference to the specified element.
